@@ -34,7 +34,6 @@ import (
 	"strings"
 	"sync"
 	"sync/atomic"
-	"testing"
 	"time"
 )
 
@@ -51,8 +50,6 @@ type SwiftServer struct {
 	// See https://golang.org/pkg/sync/atomic/#pkg-note-BUG for more details.
 	reqId int64
 	sync.RWMutex
-	t        *testing.T
-	mu       sync.Mutex
 	Listener net.Listener
 	AuthURL  string
 	URL      string
@@ -145,7 +142,6 @@ type container struct {
 	sync.RWMutex
 	metadata
 	name    string
-	ctime   time.Time
 	objects map[string]*object
 }
 
@@ -188,15 +184,6 @@ type containerResource struct {
 	container *container // non-nil if the container already exists.
 }
 
-var responseParams = map[string]bool{
-	"content-type":        true,
-	"content-language":    true,
-	"expires":             true,
-	"cache-control":       true,
-	"content-disposition": true,
-	"content-encoding":    true,
-}
-
 func fatalf(code int, codeStr string, errf string, a ...interface{}) {
 	panic(&swiftError{
 		statusCode: code,
@@ -319,9 +306,15 @@ func (r containerResource) get(a *action) interface{} {
 	} else {
 		for _, item := range objects {
 			if obj, ok := item.(*object); ok {
-				a.w.Write([]byte(obj.name + "\n"))
+				_, err := a.w.Write([]byte(obj.name + "\n"))
+				if err != nil {
+					fatalf(500, "WriteFailed", "Write failed")
+				}
 			} else if subdir, ok := item.(Subdir); ok {
-				a.w.Write([]byte(subdir.Subdir + "\n"))
+				_, err := a.w.Write([]byte(subdir.Subdir + "\n"))
+				if err != nil {
+					fatalf(500, "WriteFailed", "Write failed")
+				}
 			}
 		}
 		return nil
@@ -398,7 +391,12 @@ func (r containerResource) put(a *action) interface{} {
 			if err != nil {
 				fatalf(400, "TODO", "Invalid tar.gz")
 			}
-			defer gzr.Close()
+			defer func() {
+				err := gzr.Close()
+				if err != nil {
+					fatalf(400, "CloseFailed", "Close failed")
+				}
+			}()
 			reader = tar.NewReader(gzr)
 		case "tar.bz2":
 			bzr := bzip2.NewReader(dataReader)
@@ -660,7 +658,10 @@ func (objr objectResource) get(a *action) interface{} {
 	} else if value, ok := obj.meta["X-Static-Large-Object"]; ok && value[0] == "True" && a.req.URL.Query().Get("multipart-manifest") != "get" {
 		var segments []io.Reader
 		var segmentList []segment
-		json.Unmarshal(obj.data, &segmentList)
+		err := json.Unmarshal(obj.data, &segmentList)
+		if err != nil {
+			fatalf(400, "BadParameters", "Unmarshal failed.")
+		}
 		cursor := 0
 		size := 0
 		sum := md5.New()
@@ -769,7 +770,10 @@ func (objr objectResource) put(a *action) interface{} {
 		a.req.Header.Set("X-Static-Large-Object", "True")
 
 		var segments []segment
-		json.Unmarshal(data, &segments)
+		err := json.Unmarshal(data, &segments)
+		if err != nil {
+			fatalf(400, "BadParameters", "Unmarshal failed.")
+		}
 		for i := range segments {
 			segments[i].Name = "/" + segments[i].Path
 			segments[i].Path = ""
@@ -901,7 +905,10 @@ func (objr objectResource) copy(a *action) interface{} {
 
 func (s *SwiftServer) serveHTTP(w http.ResponseWriter, req *http.Request) {
 	// ignore error from ParseForm as it's usually spurious.
-	req.ParseForm()
+	err := req.ParseForm()
+	if err != nil {
+		fatalf(400, "BadParameters", "Parse form failed.")
+	}
 
 	if fn := s.override[req.URL.Path]; fn != nil {
 		originalRW := w
@@ -954,8 +961,8 @@ func (s *SwiftServer) serveHTTP(w http.ResponseWriter, req *http.Request) {
 				_, _ = rand.Read(r)
 				id := fmt.Sprintf("%X", r)
 				w.Header().Set("X-Storage-Url", s.URL+"/AUTH_"+username)
-				w.Header().Set("X-Auth-Token", "AUTH_tk"+string(id))
-				w.Header().Set("X-Storage-Token", "AUTH_tk"+string(id))
+				w.Header().Set("X-Auth-Token", "AUTH_tk"+id)
+				w.Header().Set("X-Storage-Token", "AUTH_tk"+id)
 				s.Sessions[id] = &session{
 					username: username,
 				}
@@ -1016,7 +1023,6 @@ func (s *SwiftServer) serveHTTP(w http.ResponseWriter, req *http.Request) {
 		if !ok {
 			s.RUnlock()
 			panic(notAuthorized())
-			return
 		}
 
 		a.user = s.Accounts[session.username]
@@ -1046,9 +1052,15 @@ func (s *SwiftServer) serveHTTP(w http.ResponseWriter, req *http.Request) {
 		} else {
 			switch r := resp.(type) {
 			case string:
-				w.Write([]byte(r))
+				_, err := w.Write([]byte(r))
+				if err != nil {
+					fatalf(500, "WriteFailed", "Write failed.")
+				}
 			default:
-				w.Write(resp.([]byte))
+				_, err := w.Write(resp.([]byte))
+				if err != nil {
+					fatalf(500, "WriteFailed", "Write failed.")
+				}
 			}
 		}
 	}
@@ -1195,7 +1207,10 @@ func (rootResource) get(a *action) interface{} {
 				Name:  container.name,
 			})
 		} else {
-			a.w.Write([]byte(container.name + "\n"))
+			_, err := a.w.Write([]byte(container.name + "\n"))
+			if err != nil {
+				fatalf(500, "WriteFailed", "Write failed.")
+			}
 		}
 	}
 
@@ -1275,7 +1290,10 @@ func (r rootResource) delete(a *action) interface{} {
 		}
 
 		resp := fmt.Sprintf("Number Deleted: %d\nNumber Not Found: %d\nErrors: \nResponse Status: 200 OK\n", nb, notFound)
-		a.w.Write([]byte(resp))
+		_, err = a.w.Write([]byte(resp))
+		if err != nil {
+			fatalf(500, "WriteFailed", "Write failed.")
+		}
 		return nil
 	}
 
@@ -1310,13 +1328,15 @@ func NewSwiftServer(address string) (*SwiftServer, error) {
 		Containers: make(map[string]*container),
 	}
 
-	go http.Serve(l, http.HandlerFunc(func(w http.ResponseWriter, req *http.Request) {
-		server.serveHTTP(w, req)
-	}))
+	go func() {
+		_ = http.Serve(l, http.HandlerFunc(func(w http.ResponseWriter, req *http.Request) {
+			server.serveHTTP(w, req)
+		}))
+	}()
 
 	return server, nil
 }
 
 func (srv *SwiftServer) Close() {
-	srv.Listener.Close()
+	_ = srv.Listener.Close()
 }
